# -*- coding: utf-8 -*-
"""
% Project           : Iterative methods and preconditioning for large 
%			and sparse linear systems with applications
%
% Program name      : sparse_mv.py
%
% Author            : F. Durastante -- fdurastante@uninsubria.it
%
% Date created      : 28 July 2017
%
% Purpose           : Benchmarking differente sparse storage formats form 
%			matrix vector products.
%
% Revision History  :
%
% Date        Author      	Ref    Revision (Date in DD/MM/YYYY format) 
% 28/07/2017  F. Durastante     1      File Created
%
"""

import numpy as np
import time as time
from scipy import sparse
from pylab import *
import matplotlib.pyplot  as pyplot

i = 0;
tcsr = np.zeros(20);
tcoo = np.zeros(20);
tcsc = np.zeros(20);
tdia = np.zeros(20);
tbsr = np.zeros(20);

for n in range(10,210,10):
    nx, ny = n,n
    N  = nx*ny
    main_diag = np.ones(N)*-4.0
    side_diag = np.ones(N-1)
    side_diag[np.arange(1,N)%4==0] = 0
    up_down_diag = np.ones(N-3)
    diagonals = [main_diag,side_diag,side_diag,up_down_diag-2,up_down_diag+2]
    Acsr = sparse.diags(diagonals, [0, -1, 1,nx,-nx], format="csr")
    Acoo = Acsr.tocoo()
    Acsc = Acsr.tocsc()
    Adia = Acsr.todia()
    Absr = Acsr.tobsr()
    x = np.ones(N)
    x = x.transpose()
    
    startcsr = time.clock()
    y = Acsr.multiply(x)
    endcsr=time.clock()
    tcsr[i] = endcsr-startcsr
    
    startcsr = time.clock()
    y = Acoo.multiply(x)
    endcsr=time.clock()
    tcoo[i] = endcsr-startcsr
    
    startcsr = time.clock()
    y = Acsc.multiply(x)
    endcsr=time.clock()
    tcsc[i] = endcsr-startcsr
    
    startcsr = time.clock()
    y = Adia.multiply(x)
    endcsr=time.clock()
    tdia[i] = endcsr-startcsr
    
    startcsr = time.clock()
    y = Absr.multiply(x)
    endcsr=time.clock()
    tbsr[i] = endcsr-startcsr
    
    i = i+1

fig = pyplot.figure()
pyplot.plot(range(10,210,10),tcoo,'k^',label = 'COO')
pyplot.plot(range(10,210,10),tcsr,'ks',label = 'CSR')
pyplot.plot(range(10,210,10),tcsc,'kp',label = 'CSC')
pyplot.plot(range(10,210,10),tdia,'kx',label = 'DIA')
pyplot.plot(range(10,210,10),tbsr,'ko',label = 'BSR')
pyplot.yscale('log')
pyplot.legend(loc='best',numpoints=1)
pyplot.ylabel('T(s)')
pyplot.xlabel('n')
savefig('mat_vec_prod.eps', dpi = 300, format = 'eps')
