%**********************************************************************;
% Project           : Iterative methods and preconditioning for large 
%			and sparse linear systems with applications
%
% Program name      : steepestdescent.m
%
% Author            : F. Durastante -- fdurastante@uninsubria.it
%
% Date created      : 28 July 2017
%
% Purpose           : Gradient descent method for the solution of Ax = b
%
% Revision History  :
%
% Date        Author      	Ref    Revision (Date in DD/MM/YYYY format) 
% 28/07/2017  F. Durastante     1      File Created
%
%**********************************************************************;
function [ x,flag,relres,iter,res ] = steepestdescent( A, b, tol, maxit )
%STEEPESTDESCENT Gradient descent method for the solution of a linear
%system.
%   A Matrix of the linear system
%   b Right--hand side
%   tol Tolerance on the residual
%   maxit Maximum number of iteration permitted

x = zeros(size(b));
res = zeros(maxit,1);
r = b - A * x;
res(1) = norm(r,2);

for i=2:maxit
     Ar = A * r; 
	 alpha = r'*r / ( r' * Ar);
	 x = x + alpha * r;
	 r = r - alpha* Ar;
     res(i) = norm(r);
     if res(i) <= tol
         iter = i;
         res(i+1:end) = [];
         relres = res(end);
         flag = 0;
         break;
     end
end

if i==maxit
    iter = maxit;
    flag = 1;
    relres = res(end);
end

end
          
