function [Z,p] = ainv_step(A,tolZ)
%AINV Compute the Approximate Inverse with Biconjugation Algorithm
%
% If A is non symmetric and positive definite then:
% [Z,p] = ainv(A,0);
% [W,p] = ainv(A',0);
% D=spdiags(p,0,n,n);
% Therefore
% norm(B-W*inv(D)*Z')==0
%
% (c) 2003-2004-2010 Daniele Bertaccini, Universita' di Roma Tor Vergata
% http://www.mat.uniroma2.it/bertaccini
%
% per A SIMMETRICA E DEF. POS:
% this function computes the 
% sparse approximate inverse in factorized 
% form, given by A=ZD^{-1}Z' 
%  ||inv(A)-Z*diag(1./pppp)*Z'||=0 in infinite precision, tolZ=0,
%
% and the L*D*(L*D)' factorization of A (A=L*D*(L*D)' if tol=0)
% NOW is GOOD EVEN IF A IS COMPLEX
%
% A is assumed to be spd.
% parameters
%   input:  A   = the matrix. Currently, A is assumed to be spd.
%           tol = drop tolerance for the elements of Z
%                 a good choice is tol=10^{-1}
%   output: Z =>  For tol=0, A^{-1} = Z*D*Z'
%           p   = vector of the diagonal elements of D
%

[~,n] = size(A);
Z = speye(n);
p = zeros(n,1);
J = 1;
figure(1);
for i = 1:n
  k = find(Z(:,i));
  v = A(:,k)*Z(k,i);
  p(i:n) = v.'*Z(:,i:n);
  u = (1/p(i))*Z(:,i); 
  for j=i+1:n 
    ZT=Z(:,j)-p(j)*u;
    logind=abs(ZT)<tolZ;
    ZT(logind)=0;
    Z(:,j)=ZT;
  end
  if(i==25 || i==155 || i==285 || i==415 || i==545 || i==675)
      subplot(3,2,J)
      spy(Z,'k');
      title(['$',sprintf('i=%d',i),'$'],'Interpreter','LaTeX');
      J = J+1;
  end
end
