%**********************************************************************;
% Project           : Iterative methods and preconditioning for large 
%			and sparse linear systems with applications
%
% Program name      : ichol0.m
%
% Author            : F. Durastante -- fdurastante@uninsubria.it
%
% Date created      : 28 July 2017
%
% Purpose           : Incomplete Cholesky Factorization, illustrative
%			purpose only
%
% Revision History  :
%
% Date        Author      	Ref    Revision (Date in DD/MM/YYYY format) 
% 28/07/2017  F. Durastante     1      File Created
%
%**********************************************************************;
function A = ichol0(A)
% ICHOL0 compute the IC(0) factorization for the square matrix A. This
% routine is for illustrative purposes. See Example 3.1 in the book.

n = size(A,1);

for k=1:n
    A(k,k) = sqrt(A(k,k));
    for i=(k+1):n
        if (A(i,k) ~= 0)
            A(i,k) = A(i,k)/A(k,k);
        end
    end
    for j=(k+1):n
        for i=j:n
            if (A(i,j) ~= 0)
                A(i,j) = A(i,j)-A(i,k)*A(j,k);
            end
        end
    end
end

A = tril(A);
end
