function match = mmaker(A, matchID, cheapID)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Usage:
%   [match] = mmaker(A)
%   [match] = mmaker(A, matchID)
%   [match] = mmaker(A, matchID, cheapID)
%   [match] = mmaker(A, matchID, cheapID, relabel_period)
%  
%   INPUT ARGUMENTS
%   A: a sparse matrix
%   matchID: id of match algo (1-9)
%   		1: DFS based
%  		2: BFS based
%  		3: MC21 (DFS + lookahead)
%  		4: PF (Pothen and Fan' algorithm)
%  		5: PF+ (PF + fairness)
%  		6: HK (Hopcroft and Karp's algorithm)
%  		7: HK-DW (Duff-Wiberg implementation of HK)
%  		8: ABMP (Alt et al.'s algorithm)
%  		9: ABMP-BFS (ABMP + BFS)
%  	   10: PR-FIFO-FAIR (DEFAULT)
%  
%   cheapID: id of cheap algo (1-4)
%  		1: Simple Greedy
%  		2: Karp-Sipser
%  		3: Random Karp-Sipser (DEFAULT)
%  		4: Minimum Degree (two-sided)
%  
%   relabel_period: used only when matchID = 10. Otherwise it is ignored.
%        For the PR based algorithm, a global relabeling is started after
%        every (m+n) x 'relabel_period' pushes where m and
%   		n are the number of rows and columns of the matrix. Default is 1.
%   			-1: for a global relabeling after every m pushes
%   			-2: for a global relabeling after every n pushes
%   		Other than these two, non-positive values are not allowed.
%  
%   OUTPUT ARGUMENTS
%   match: match of rows match(r) = c for r = 1,...,m;
%          if c = -1, then row r is not matched; else column c and row r
%          are matched.
%  
%   Please see the reports:
%   	"I. S. Duff, K. Kaya and B. Ucar.
%   	'Design, Implementations and Analysis of Maximum Transversal Algorithms'
%   	CERFACS Tech.  Report TR/PA/10/76, October, 2010."
%  
%   	"K. Kaya, J. Langguth, F. Manne and B. Ucar.
%   	'Experiments on Push-Relabel-based Maximum Cardinality Matching 
%        Algorithms for Bipartite Graphs'
%   	CERFACS Tech.  Report TR/PA/11/33, May, 2011."
%  
%   for more details and cite them if you use the codes.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
help mmaker

error('mexFunction could not found. Please compile mmaker.c\n');

end%function mmaker
