%% TEST FOR THE PRECONDITIONING OF NOT HERMITIAN MATRICES
% Test for some ILU type preconditioner in a not Hermitian case obtained
% from the finite difference discretization of the advection-diffusion
% equation:
% -k div(grad(u))-nu grad(u) = f, for (x,y) \in [0,1]x[0,1]
%  u(x,0) = u(x,1) = u(1,y) = u(0,y) = 0 
% the code permits the usage of different number of grid points in the two
% side of the discretizations.
% ----------------------------------------------------------------------- %
clear;
clc;
% ----------------------------------------------------------------------- %

N = input('Number of grid points on the x axis: ');
M = input('Number of grid points on the y axis: ');
K = input('Diffusion Coefficient: ');
nu = input('Convection Velocity: ');

f = @(x,y) sin(x).*cos(y);

x = linspace(0,1,N);
y = linspace(0,1,M);
[X,Y] = meshgrid(x,y);
eN = ones(N,1); hx = 1/(N-1);
eM = ones(M,1); hy = 1/(M-1);
Tx = spdiags([-eN,2*eN,-eN]./hx,-1:1,N,N);
Ty = spdiags([-eM,2*eM,-eM]./hy,-1:1,M,M);
Bx = spdiags([-eN,eN]./(2*hx),[-1,1],N,N);
By = spdiags([-eM,eM]./(2*hy),[-1,1],M,M);

A = kron(K*Tx + nu*Bx,speye(M))+ kron(speye(N),K*Ty + nu*By);
F = reshape(f(X,Y),N*M,1);

tol = 1e-6;

tic; [Xg1,FLAGg1,~,ITERg1] = gmres(A,F,[],tol,M*N); timeg1 = toc;
tic; [Xg2,FLAGg2,~,ITERg2]= gmres(A,F,20,tol,M*N); timeg2 = toc;
tic; [Xb1,FLAGb1,~,ITERb1] = bicgstab(A,F,tol,M*N); timeb1 = toc;

opts.type='nofill';
opts.milu='row';
[L1,U1] = ilu(A,opts);

opts.type='crout';
opts.milu='row';
opts.droptol=input('Insert threshold for the ILUT algorithm: ');
[L2,U2] = ilu(A,opts);

figure(1)
subplot(2,2,1); spy(A,'k'); axis square; title('Pattern','interpreter','LaTeX');
subplot(2,2,2); eigA = eig(full(A)); semilogx(real(eigA),imag(eigA),'k+'); axis square; title('$\sigma(A_h)$','interpreter','LaTeX');
subplot(2,2,3); eigPA1 = eig(full(U1\(L1\A))); semilogx(real(eigPA1),imag(eigPA1),'k+'); axis square; title('ILU(0) $\sigma(P^{-1}A_h)$','interpreter','LaTeX');
subplot(2,2,4); eigPA2 = eig(full(U2\(L2\A))); semilogx(real(eigPA2),imag(eigPA2),'k+'); axis square; title('ILUTP(1e-2) $\sigma(P^{-1}A_h)$','interpreter','LaTeX');

tic; [Xg3,FLAGg3,~,ITERg3] = gmres(A,F,[],tol,M*N,L1,U1); timeg3 = toc;
tic; [Xg4,FLAGg4,~,ITERg4]= gmres(A,F,20,tol,M*N,L1,U1); timeg4 = toc;
tic; [Xb2,FLAGb2,~,ITERb2] = bicgstab(A,F,tol,M*N,L1,U1); timeb2 = toc;
tic; [Xg5,FLAGg5,~,ITERg5] = gmres(A,F,[],tol,M*N,L2,U2); timeg5 = toc;
tic; [Xg6,FLAGg6,~,ITERg6]= gmres(A,F,20,tol,M*N,L2,U2); timeg6 = toc;
tic; [Xb3,FLAGb3,~,ITERb3] = bicgstab(A,F,tol,M*N,L2,U2); timeb3 = toc;

fprintf(...
    '%d & %d & %d & %1.4f & %d & %1.4f & %d & %1.4f & %d & %1.4f & %d & %1.4f & %d & %1.4f & %d & %1.4f & %d & %1.4f & %d & %1.4f \n',...
    N,M,ITERg1(2),timeg1,ITERg2(1)*20+ITERg2(2),timeg2,2*ITERb1,timeb1,...
    ITERg3(2),timeg3,ITERg4(1)*20+ITERg4(2),timeg4,2*ITERb2,timeb2,...
    ITERg5(2),timeg5,ITERg6(1)*20+ITERg6(2),timeg6,2*ITERb3,timeb3);
