%% TEST FOR THE PRECONDITIONING OF SOME HERMITIAN MATRICES
% This codes depends on the UFGET code from:
% http://www.cise.ufl.edu/research/sparse/matrices/
% to obtain the test matrices

clear;
clc;

addpath('./UFget/'); % Set the path to the UFGET package!

test_problem = [219,221,222,223,2,3,1,159];

TOL = 1e-6;

for number = test_problem
    Problem = UFget(number);    % Downlaod the problem
    
    A = Problem.A;              % Build the linear system
    n = size(A,1);
    b = ones(n,1);
    
    MAXIT = n;
    
    tic; [X1,FLAG1,RELRES1,ITER1] = pcg(A,b,TOL,MAXIT); t1=toc;
    try
        L = ichol(A,struct('type','ict','droptol',1e-01,'michol','off'));
        fill2 = ((2*nnz(L)-n)/n^2)*100;
        tic; [X2,FLAG2,RELRES2,ITER2] = pcg(A,b,TOL,MAXIT,L,L'); t2=toc;
    catch
        ITER2 = -1;
        t2 = -1;
        fill2 = -1;
    end
    try
        L = ichol(A,struct('type','ict','droptol',1e-02,'michol','off'));
        fill3 = ((2*nnz(L)-n)/n^2)*100;
        tic; [X3,FLAG3,RELRES3,ITER3] = pcg(A,b,TOL,MAXIT,L,L'); t3=toc;
    catch
        ITER3 = -1;
        fill3 = -1;
        t3 = -1;
    end
    [Z,p] = ainv(A,1e-1);
    D = spdiags(1./p,0,n,n);
    P = @(x) Z*(D*(Z'*x));
    fill4 = ((2*nnz(Z)-n)/n^2)*100;
    tic; [X4,FLAG4,RELRES4,ITER4] = pcg(A,b,TOL,MAXIT,@(x) P(x)); t4=toc;
    [Z,p] = ainv(A,1e-2);
    D = spdiags(1./p,0,n,n);
    P = @(x) Z*(D*(Z'*x));
    fill5 = ((2*nnz(Z)-n)/n^2)*100;
    tic; [X5,FLAG5,RELRES5,ITER5] = pcg(A,b,TOL,MAXIT,@(x) P(x)); t5=toc;
    
    fprintf(...
        '%s & %d & %d & %1.4f & %d & %1.4f & %1.2f & %d & %1.4f & %1.2f & %d & %1.4f & %1.2f & %d & %1.4f & %1.2f \n',...
        Problem.name,n,ITER1,t1,ITER2,t2,fill2,ITER3,t3,fill3,ITER4,t4,fill4,ITER5,t5,fill5);
    
    
end