function [ x ] = CSCS( T,b,alpha,tol,maxit )
%CSCS Circulant-Skew-Circulant-Splitting solver. Two step solver that uses
%the CSC splitting for the Toeplitz matrix T.
%   T = Toeplitz matrix
%   b = right-hand side
%   alpha = paramater
%   tol = tolerance on the residual
%   maxit = maximum number of iteration admitted

n = size(T,1);
% [C,S] = circdecomp(T,'col');
% 
% O = (-1).^(0:-1/n:-(n-1)/n)';
% 
% L1 = C;    L1(1) = L1(1) + alpha; L1 = fft(L1);
% R1 = -S;   R1(1) = R1(1) + alpha; R1 = fft(O.*R1);
% 
% L2 = S;    L2(1) = L2(1) + alpha; L2 = fft(O.*L2);
% R2 = -C;   R2(1) = R2(1) + alpha; R2 = fft(R2);
[C,S] = circdecomp(T);
L1 = C + alpha*eye(n);
R1 = alpha*eye(n) - S;
L2 = S + alpha*eye(n);
R2 = alpha*eye(n) - C;

x = zeros(n,1);

for it=1:maxit
    %x = cprec(L1,ocircmv(R1,-1,x)+b);
    %x = ocircsol(L2,-1,circmv(R2,x)+b);
    x = L1\(R1*x + b);
    x = L2\(R2*x + b);
    
    res = norm(b - toepmv(T,x),2);
    if  res < tol
        fprintf('CSCS converged in %d iteration with last residual %e\n',...
            it,res);
        return        
    end
end

fprintf('CSCS reached maxit with last residual %e\n',res);

end

