function [ C,S ] = circdecomp( T,varargin )
%CIRCDECOMP decompose the toeplitz matrix T in its circulant and
%skew-circulant part.
%   T = Toeplitz matrix
%   C = Circulant part
%   S = Skew-circulant part
%   If opts = 'col' C,S matrix first column
   
if nargin == 1
    C = gallery('circul',0.5*[T(1,1),T(1,2:end)+T(end:-1:2,1)']);
    S = toeplitz(0.5*[T(1,1),-T(1,end:-1:2)+T(2:end,1)'],...
        0.5*[T(1,1),T(1,2:end)-T(end:-1:2,1)']);
elseif nargin == 2 && strcmp(varargin{1},'col')
    C = 0.5*[T(1,1);flipud(T(1,2:end)'+T(end:-1:2,1))];
    S = 0.5*[T(1,1);-T(1,end:-1:2)'+T(2:end,1)];
else
    error('unknown argument!');
end

