%**********************************************************************;
% Project           : Iterative methods and preconditioning for large 
%			and sparse linear systems with applications
%
% Program name      : ex_complexrealsolve.m
%
% Author            : F. Durastante -- fdurastante@uninsubria.it
%
% Date created      : 28 July 2017
%
% Purpose           : Solution of Real Block matrices formulation from 
%			Complex Matrices with ILU factorizations
%
% Revision History  :
%
% Date        Author      	Ref    Revision (Date in DD/MM/YYYY format) 
% 28/07/2017  F. Durastante     1      File Created
%
%**********************************************************************;
%% Solution with incomplete factorizations of the real equivalent formulations of a complex system

clear;
clc;

C = mmread('dwg961a.mtx');
C = C(1:705,1:705);
d = (1+1i)*C*ones(length(C),1);

A = real(C); b = real(d);
B = imag(C); c = real(d);

% Equivalente formulation
M1 = [A,-B;B,A];
M2 = [A,B;B,-A];
M3 = [B,A;A,-B];
M4 = [B,-A;A,B];
y = [b;c];

% ILU-Type preconditioner
opts.droptol = 1e-1;
opts.milu = 'row';
opts.type = 'ilutp';
opts.udiag = 1;
[L1,U1] = ilu(M1,opts);
%[L2,U2] = ilu(M2,opts);
[L3,U3] = ilu(M3,opts);
[L4,U4] = ilu(M4,opts);

X1 = gmres(@(x) realprod(A,B,x,1),y,[],1e-6,2*length(C),L1,U1);
%X2 = gmres(@(x) realprod(A,B,x,2),y,[],1e-6,2*length(C),L2,U2);
X3 = gmres(@(x) realprod(A,B,x,3),y,[],1e-6,2*length(C),L3,U3);
X4 = gmres(@(x) realprod(A,B,x,4),y,[],1e-6,2*length(C),L4,U4);

X1 = gmres(@(x) realprod(A,B,x,1),y,[],1e-6,2*length(C));
X2 = gmres(@(x) realprod(A,B,x,2),y,[],1e-6,2*length(C));
X3 = gmres(@(x) realprod(A,B,x,3),y,[],1e-6,2*length(C));
X4 = gmres(@(x) realprod(A,B,x,4),y,[],1e-6,2*length(C));
