%**********************************************************************;
% Project           : Iterative methods and preconditioning for large 
%			and sparse linear systems with applications
%
% Program name      : ex_complexrealspectra.m
%
% Author            : F. Durastante -- fdurastante@uninsubria.it
%
% Date created      : 28 July 2017
%
% Purpose           : Spectrum of Real Block matrices from Complex Matrices
%
% Revision History  :
%
% Date        Author      	Ref    Revision (Date in DD/MM/YYYY format) 
% 28/07/2017  F. Durastante     1      File Created
%
%**********************************************************************;
%% Spectral properties of the real equivalent approximation of a complex system

C = mmread('young2c.mtx');

%% Equivalent formulation
A = real(C);
B = imag(C);
M1 = [A,-B;B,A];
M2 = [A,B;B,-A];
M4 = [B,-A;A,B];

%% Computation of the eigenvalues
eigC = eig(full(C));
eigM1 = eig(full(M1));
eigM2 = eig(full(M2));
eigM4 = eig(full(M4));

%% Plot of the figure

figure(1)
plot(real(eigC),imag(eigC),'kx');
axis square
figure(2)
plot(real(eigM1),imag(eigM1),'kx');
axis square
figure(3)
plot(real(eigM2),imag(eigM2),'kx');
axis square
figure(4)
plot(real(eigM4),imag(eigM4),'kx');
axis square
