function [ y ] = ocircmv( W,omega,v )
%OCIRCMV Omega-circulant matrix-vector product
%   If the imput W is a matrix computes its eigenvalues with an FFT and
%   then uses another fft and one ifft to compute the matrix vector product
%   Otherwise it assumes that W is the vector of the eigenvalues.
%   If omega = 1 then W is a circulant matrix and the routine calls for its
%   equivalent for the circulant matrices.

if omega == 1
    y = circmv(W,v);
    return
end

n = size(W,1);
O = omega.^(0:-1/n:-(n-1)/n)';

if ismatrix(W)
    eig = fft(O.*W(1:end,1));
else
    eig = W(1:end)';
end

y = conj(O).*ifft(eig.*fft(O.*v));

if norm(imag(y))<1e-10
    y = real(y);
end

end

