# Illustrates the selection of Coarse-Fine (CF) 
# splittings in Classical AMG.
# Modified version from the original example of PyAMG

import numpy
from scipy.io import loadmat
from pyamg import ruge_stuben_solver

data = loadmat('unit_square.mat')

A = data['A']                     # matrix
V = data['vertices'][:A.shape[0]]            # vertices of each variable
E = numpy.vstack((A.tocoo().row,A.tocoo().col)).T  # edges of the matrix graph

# Use Ruge-Stuben Splitting Algorithm (use 'keep' in order to retain the splitting)
mls = ruge_stuben_solver(A, max_levels=2, max_coarse=1, CF='RS',keep=True)
print mls

# The CF splitting, 1 == C-node and 0 == F-node
splitting = mls.levels[0].splitting
C_nodes = splitting == 1
F_nodes = splitting == 0

from draw import lineplot
from pylab import figure, axis, scatter, show, spy
import matplotlib as mplt

figure(figsize=(12,12))
axis('equal')
lineplot(V, E)
coarse = scatter(V[:,0][C_nodes], V[:,1][C_nodes], c='k',marker='o', s=100.0,label='Coarse')  #plot C-nodes in red
fine = scatter(V[:,0][F_nodes], V[:,1][F_nodes], c='k',marker='v', s=100.0,label='Fine')  #plot F-nodes in blue
mplt.pyplot.legend(handles=[coarse,fine], loc=1,scatterpoints=1)
fig = mplt.pyplot.gcf()
fig.savefig('coarsening.eps', format='eps', dpi=600)
show()
figure(figsize=(12,12))
spy(A,precision=0.01,markersize=2,marker='s',c='k')
fig = mplt.pyplot.gcf()
fig.savefig('spy_coarsening.eps', format='eps', dpi=600)