# Examples of AMG as a preconditioner

import scipy
import numpy
from pyamg.gallery import poisson
from pyamg import smoothed_aggregation_solver, ruge_stuben_solver

# Create test cases
trials = []
A,B = poisson((500,500), format='csr'), None
trials.append( ('Poisson',A,B) )
   
print "Show advantages of accleration for two example problems"
choice = input('\n Input Choice:\n' + \
            '1:  Run smoothed_aggregation_solver\n' + \
            '2:  Run ruge_stuben_solver (Strenght = 0.25)\n'  + \
            '3:  Run Run ruge_stuben_solver (Strenght = 0.50)\n'+ \
            'Choice = ')
if choice == 1:
    method = smoothed_aggregation_solver
    namem = 'Smoothed Aggregation'
elif choice == 2:
    method = ruge_stuben_solver
    namem = 'Classical Solver'
    strength = 0.25
elif choice == 3:
    method = ruge_stuben_solver
    namem = 'Classical Solver'
    strength = 0.50
else:
    raise ValueError("Enter a choice of 1,2 or 3")


for name,A,B in trials:
    # Construct solver using AMG based on Smoothed Aggregation (SA)
    if choice == 1:
        mls = method(A, B=B)
    else:
        mls = method(A,strength=('classical',{'theta': strength}))
    # Display hierarchy information
    print 'Matrix: %s' % name
    print mls

    # Create random right hand side
    b = scipy.rand(A.shape[0],1)
    
    # Solve Ax=b with no acceleration ('standalone' solver)
    standalone_residuals = []
    x = mls.solve(b, tol=1e-10, accel=None, residuals=standalone_residuals)
    
    # Solve Ax=b with Conjugate Gradient (AMG as a preconditioner to CG)
    accelerated_residuals = []
    x = mls.solve(b, tol=1e-10, accel='cg', residuals=accelerated_residuals)
    
    # Compute relative residuals
    standalone_residuals  = numpy.array(standalone_residuals)/standalone_residuals[0]  
    accelerated_residuals = numpy.array(accelerated_residuals)/accelerated_residuals[0]  
    
    # Plot convergence history
    import pylab
    pylab.figure()
    if choice == 1:
        pylab.title('%s Convergence History (%s)' % (namem,name))
    else:
        pylab.title('%s Convergence History (%s) - Strength %f' % (namem,name,strength))
    pylab.xlabel('Iteration')
    pylab.ylabel('Relative Residual')
    pylab.semilogy(standalone_residuals,  label='Standalone',  linestyle='None', marker='o', c='k')
    pylab.semilogy(accelerated_residuals, label='Accelerated', linestyle='None', marker='s', c='k')
    pylab.legend()
    fig = pylab.gcf()
    fig.savefig('convergence'+ str(choice) +'.eps', format='eps', dpi=600)
    pylab.show()