function t = toepgenerator(n,fchoice)
% kern generates the first column of Toeplitz matrix T
% depending on fchoice.
% n: the size of the Toeplitz matrix T;
% fchoice = 1: example for Chapter 2;
% fchoice = 2 and 3: examples for Chapter 3;
% fchoice = 4 to 9: examples for Chapter 4;
% t: the first column of the Toeplitz matrix.
t = zeros(1,n);
if fchoice == 1
    t(1) = 2;
    t(2:n) = (1+sqrt(-1))./((2:n).^1.1);
elseif fchoice == 2
    i = sqrt(-1);
    t(1) = 4.2;
    k = 1:n-1;
    t(2:n) = (exp(i*k.*log(k)))./k;
elseif fchoice == 3
    i = sqrt(-1);
    t(1) = 6.5;
    k = 1:n-1;
    t(2:n) = (exp(i*k.*log(k)))./sqrt(k);
elseif fchoice == 4 % f(x) = x^4+1;
    pi2 = pi*pi;
    k = (1:n-1).^2;
    t(1) = pi2*pi2/5 + 1;
    t(2:n) = (4*pi2-24./k)./k;
    t(2:2:n) = -t(2:2:n);
elseif fchoice == 5 % f(x) = |x|^3+0.01;
    k = (1:n-1).^2;
    tp1 = ones(1,n-1); tp1(1:2:n-1) = -1;
    tp2 = zeros(1,n-1); tp2(1:2:n-1) = 12;
    t(1) = 1/4*pi^3+0.01;
    t(2:n) = ((3*pi)*tp1)./k+(tp2/pi)./(k.^2);
    t(2:n) = (3*pi*tp1+(tp2/pi)./k)./k;
elseif fchoice == 6 % f(x) = x^2;
    t(1) = pi*pi/3;
    t(2:n) = 2./((1:n-1).^2);
    t(2:2:n) = -t(2:2:n);
elseif fchoice == 7 % f(x) = x^2(pi^4-x^4);
    pi2 = pi*pi; pi4 = pi2*pi2;
    tp = 194.8181820680048-6*pi4;
    k = (1:n-1).^2;
    t(1) = -1/7*pi2*pi4+32.46969701133414*pi2;
    t(2:n) = ((-720./k+pi2*120)./k+tp)./k;
    t(2:2:n) = -t(2:2:n);
elseif fchoice == 8 % f(x) = x^4;
    pi2 = pi*pi;
    k = (1:n-1).^2;
    t(1)= pi2*pi2/5;
    t(2:n) = (4*pi2-24./k)./k;
    t(2:2:n) = -t(2:2:n);
elseif fchoice == 9 % f(x) = x^4(pi^2-x^2);
    pi2 = pi*pi;
    pi4 = pi2*pi2;
    k = (1:n-1).^2;
    t(1)= (-1/7*pi2+1.97392088021787)*pi4;
    c1 = 39.47841760435743*pi2-6*pi4;
    c2 = 120*pi2-236.8705056261446;
    t(2:n) = ((-720./k+c2)./k+c1)./k;
    t(2:2:n) = -t(2:2:n);
elseif fchoice == 10 % f(x) = |x|^3;
    k = (1:n-1).^2;
    tp1 = ones(1,n-1); tp1(1:2:n-1) = -1;
    tp2 = zeros(1,n-1); tp2(1:2:n-1) = 12;
    t(1)= 1/4*pi^3;
    t(2:n) = ((3*pi)*tp1)./k+(tp2/pi)./(k.^2);
    t(2:n) = (3*pi*tp1+(tp2/pi)./k)./k;
elseif fchoice == 11 % f(x) = sigma-0.3862;
    t(1)= 0.6138;
    m = min(n,1024);
    t(2:m) = 1./(1+(1:m-1));
end