% Test for the Toeplitz Multigrid Method (1D)

clear;
clc;

fprintf('Select the problem:\n');
fprintf('1) f(x) = 2-2 cos(x)\n');
fprintf('2) f(x) = 6-6 cos(x)\n');
fprintf('3) f(x) =  x^4 + 1\n');
fprintf('4) f(x) = x^4 + x^2 + 1\n');
fprintf('5) f(x) = x^2\n');
fprintf('6) f(x) = x^4\n');
fprintf('7) f(x) = |x|^3\n');
problem = input('problem = ');
for k = 6:12
    n = 2^k - 1;    % Ad ogni livello ho sempre potenze dispari
    
    if problem == 1
        t = zeros(1,n);
        t(1:2) = [2 -1];
        maxf = 4;
    elseif problem == 2
        t = zeros(1,n);
        t(1:3) = [6 -4 1]/8;
        maxf = 3/2;
    elseif problem == 3
        t = zeros(n,1);
        t(1) = 1+(pi^4)/5;
        for j=2:n
            t(j) = (4*(-1)^(j-1))*(-6 + (j-1)^2*pi^2)/((j-1)^4);
        end
        maxf = pi^4+1;
    elseif problem == 4
        t = zeros(n,1);
        t(1) = 1 + pi^(2)/3 + pi^(4)/5;
        for j=2:n
            t(j) = (2*(-1)^(j-1))*(-12 + (j-1)^2*(1+2*pi^2))/((j-1)^4);
        end
        maxf = 1+pi^2+pi^4;
    elseif problem == 5
        t = toepgenerator(n,6);
        maxf = pi^2;
    elseif problem == 6
        t = toepgenerator(n,8);
        maxf = pi^4;
    elseif problem == 7
        t = toepgenerator(n,10);
        maxf = pi^3;
    else
        error('Problem unrecognized!');
    end
    
    % Building matrix and reference solution:
    tt = linspace(0,pi,n)';
    x = sin(4*tt) + 2*sin(50*tt) - 1; % Building frequences
    A = sparse(toeplitz(t));
    b = A*x;
    
    %The Multigrid Method
    %a = [2,1];
    a = [6 4 1];
    %a = [70,28,1,8,56];
    xtrue = x;
    omega1 = t(1)/maxf;
    omega2 = 2*t(1)/maxf;
    v1 = 2;
    v2 = 2;
    
    gamma = 2;
    numiter = 1000;
    tol = 1e-6;
    x0 = zeros(size(x));
    tic;
    [x,err] = mgm_main(A,b,x0,numiter,tol,a,v1,v2,omega1,omega2,gamma,xtrue);
    time = toc;
    fprintf('%d \n',length(err));
    figure(1)
    semilogy(err,'--ko');
    title(sprintf('Converged in %d it',length(err)));
    xlabel('Iteration');
    ylabel('Error');
end