function [ y ] = toepmv( t,x )
%TOEPMV Computes the fast Toeplitz matrix-vector product by embedding
%   If T is a matrix takes its last column and produces the circulant
%   embedding by computing the eigenvalues of the latter. If used inside a
%   routine that performs lots of matrix-vector product the computation
%   inside the if cycle is better done outside this routine. In this way
%   you gain an FFT per iteration.

if ismatrix(t) && size(t,1) == size(t,2) % If you pass a matrix compute 
                                         % its eigenvalues
    n = size(t,1); 
    t = [t(:,1);0;t(1,n:-1:2)'];
    t = fft(t);
elseif size(t,2) ~= 1       % Check that the vector is oriented in a
    t = t.';                % consistent way for the following computations
end

n = length(x);              % Now that the eigenvalues are at hand it does
y = zeros(2*n,1);           % the computation via embedding. We could have
y(1:n) = x;                 % used the circmv function, nevertheless we
y = ifft(fft(y).*t);        % didn't to reduce the amount of memory
y = y(1:n);                 % transfer inside this routine.

if norm(imag(y)) < 1.0e-14  % Check for eventual numerical error
    y = real(y);
end

end

