(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 11.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[     17602,        543]
NotebookOptionsPosition[     16448,        500]
NotebookOutlinePosition[     16785,        515]
CellTagsIndexPosition[     16742,        512]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"GeneralizedHilbertMatrix", "[", 
    RowBox[{"p_", ",", "n_"}], "]"}], " ", ":=", " ", 
   RowBox[{"Table", "[", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"p", "+", "i", "+", "j", "-", "1"}], ")"}], 
      RowBox[{"-", "1"}]], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", "n"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "1", ",", "n"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"GeneralizedHilbertInverse", "[", 
    RowBox[{"p_", ",", "n_"}], "]"}], ":=", " ", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      FractionBox[
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], 
        RowBox[{"i", "+", "j"}]], 
       RowBox[{"p", "+", "i", "+", "j", "-", "1"}]], 
      FractionBox[
       RowBox[{"(", 
        RowBox[{
         UnderoverscriptBox["\[Product]", 
          RowBox[{"k", "=", "0"}], 
          RowBox[{"n", "-", "1"}]], 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"p", "+", "i", "+", "k"}], ")"}], 
          RowBox[{"(", 
           RowBox[{"p", "+", "j", "+", "k"}], ")"}]}]}], ")"}], 
       RowBox[{
        RowBox[{"Factorial", "[", 
         RowBox[{"i", "-", "1"}], "]"}], 
        RowBox[{"Factorial", "[", 
         RowBox[{"n", "-", "i"}], "]"}], 
        RowBox[{"Factorial", "[", 
         RowBox[{"j", "-", "1"}], "]"}], 
        RowBox[{"Factorial", "[", 
         RowBox[{"n", "-", "j"}], "]"}]}]]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", "n"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "1", ",", "n"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"NormSum", "[", 
    RowBox[{"p_", ",", "j_", ",", "n_"}], "]"}], ":=", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"PolyGamma", "[", 
      RowBox[{"0", ",", 
       RowBox[{"j", "+", "p"}]}], "]"}]}], "+", 
    RowBox[{"PolyGamma", "[", 
     RowBox[{"0", ",", 
      RowBox[{"j", "+", "n", "+", "p"}]}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"NormInvSum", "[", 
     RowBox[{"p_", ",", "j_", ",", "n_"}], "]"}], ":=", " ", 
    FractionBox[
     RowBox[{
      RowBox[{"Gamma", "[", 
       RowBox[{"1", "+", "n", "+", "p"}], "]"}], " ", 
      RowBox[{"Gamma", "[", 
       RowBox[{"j", "+", "n", "+", "p"}], "]"}], " ", 
      RowBox[{"HypergeometricPFQRegularized", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"1", "-", "n"}], ",", 
          RowBox[{"j", "+", "p"}], ",", 
          RowBox[{"1", "+", "n", "+", "p"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"1", "+", "p"}], ",", 
          RowBox[{"1", "+", "j", "+", "p"}]}], "}"}], ",", 
        RowBox[{"-", "1"}]}], "]"}]}], 
     RowBox[{
      RowBox[{"Gamma", "[", "j", "]"}], " ", 
      RowBox[{"Gamma", "[", "n", "]"}], " ", 
      RowBox[{"Gamma", "[", 
       RowBox[{"1", "-", "j", "+", "n"}], "]"}]}]]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{"H", " ", "=", " ", 
  RowBox[{"GeneralizedHilbertMatrix", "[", 
   RowBox[{"1", ",", "4"}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"Hinv", " ", "=", " ", 
  RowBox[{"GeneralizedHilbertInverse", "[", 
   RowBox[{"1", ",", "4"}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"NormInvSum", "[", 
    RowBox[{"1", ",", "j", ",", "4"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", "1", ",", "4"}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{"Abs", "[", 
      RowBox[{"Hinv", "[", 
       RowBox[{"[", 
        RowBox[{"i", ",", "j"}], "]"}], "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", "4"}], "}"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", "1", ",", "4"}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"NormSum", "[", 
    RowBox[{"1", ",", "j", ",", "4"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", "1", ",", "4"}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{"Abs", "[", 
      RowBox[{"H", "[", 
       RowBox[{"[", 
        RowBox[{"i", ",", "j"}], "]"}], "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "1", ",", "4"}], "}"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", "1", ",", "4"}], "}"}]}], "]"}], "\[IndentingNewLine]", 

 RowBox[{
  RowBox[{"MatrixForm", "[", 
   RowBox[{"Hinv", ".", "H"}], "]"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.6980560333447647`*^9, 3.698056034680273*^9}, {
   3.698056072244433*^9, 3.698056434812018*^9}, 3.698056464830089*^9, {
   3.698056525616601*^9, 3.6980565496141977`*^9}, {3.698057306371036*^9, 
   3.698057567218713*^9}, {3.698057603967992*^9, 3.698057614222517*^9}, {
   3.698057949066866*^9, 3.698057979221036*^9}, {3.698058340024974*^9, 
   3.698058372380582*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     FractionBox["1", "2"], ",", 
     FractionBox["1", "3"], ",", 
     FractionBox["1", "4"], ",", 
     FractionBox["1", "5"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     FractionBox["1", "3"], ",", 
     FractionBox["1", "4"], ",", 
     FractionBox["1", "5"], ",", 
     FractionBox["1", "6"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     FractionBox["1", "4"], ",", 
     FractionBox["1", "5"], ",", 
     FractionBox["1", "6"], ",", 
     FractionBox["1", "7"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     FractionBox["1", "5"], ",", 
     FractionBox["1", "6"], ",", 
     FractionBox["1", "7"], ",", 
     FractionBox["1", "8"]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.69805638489257*^9, 3.6980564351468277`*^9}, 
   3.698056467600115*^9, {3.69805652932196*^9, 3.698056552063376*^9}, 
   3.698057461926855*^9, {3.698057547129973*^9, 3.6980575720599117`*^9}, 
   3.698057614928856*^9, 3.69805798427413*^9, 3.698058380165234*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"200", ",", 
     RowBox[{"-", "1200"}], ",", "2100", ",", 
     RowBox[{"-", "1120"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "1200"}], ",", "8100", ",", 
     RowBox[{"-", "15120"}], ",", "8400"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2100", ",", 
     RowBox[{"-", "15120"}], ",", "29400", ",", 
     RowBox[{"-", "16800"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "1120"}], ",", "8400", ",", 
     RowBox[{"-", "16800"}], ",", "9800"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{{3.69805638489257*^9, 3.6980564351468277`*^9}, 
   3.698056467600115*^9, {3.69805652932196*^9, 3.698056552063376*^9}, 
   3.698057461926855*^9, {3.698057547129973*^9, 3.6980575720599117`*^9}, 
   3.698057614928856*^9, 3.69805798427413*^9, 3.6980583801665277`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"4620", ",", "32820", ",", "63420", ",", "36120"}], "}"}]], "Output",\

 CellChangeTimes->{{3.69805638489257*^9, 3.6980564351468277`*^9}, 
   3.698056467600115*^9, {3.69805652932196*^9, 3.698056552063376*^9}, 
   3.698057461926855*^9, {3.698057547129973*^9, 3.6980575720599117`*^9}, 
   3.698057614928856*^9, 3.69805798427413*^9, 3.698058380167418*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"4620", ",", "32820", ",", "63420", ",", "36120"}], "}"}]], "Output",\

 CellChangeTimes->{{3.69805638489257*^9, 3.6980564351468277`*^9}, 
   3.698056467600115*^9, {3.69805652932196*^9, 3.698056552063376*^9}, 
   3.698057461926855*^9, {3.698057547129973*^9, 3.6980575720599117`*^9}, 
   3.698057614928856*^9, 3.69805798427413*^9, 3.698058380168118*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox["77", "60"], ",", 
   FractionBox["19", "20"], ",", 
   FractionBox["319", "420"], ",", 
   FractionBox["533", "840"]}], "}"}]], "Output",
 CellChangeTimes->{{3.69805638489257*^9, 3.6980564351468277`*^9}, 
   3.698056467600115*^9, {3.69805652932196*^9, 3.698056552063376*^9}, 
   3.698057461926855*^9, {3.698057547129973*^9, 3.6980575720599117`*^9}, 
   3.698057614928856*^9, 3.69805798427413*^9, 3.698058380168808*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox["77", "60"], ",", 
   FractionBox["19", "20"], ",", 
   FractionBox["319", "420"], ",", 
   FractionBox["533", "840"]}], "}"}]], "Output",
 CellChangeTimes->{{3.69805638489257*^9, 3.6980564351468277`*^9}, 
   3.698056467600115*^9, {3.69805652932196*^9, 3.698056552063376*^9}, 
   3.698057461926855*^9, {3.698057547129973*^9, 3.6980575720599117`*^9}, 
   3.698057614928856*^9, 3.69805798427413*^9, 3.69805838016958*^9}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "0", "0"},
     {"0", "1", "0", "0"},
     {"0", "0", "1", "0"},
     {"0", "0", "0", "1"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.69805638489257*^9, 3.6980564351468277`*^9}, 
   3.698056467600115*^9, {3.69805652932196*^9, 3.698056552063376*^9}, 
   3.698057461926855*^9, {3.698057547129973*^9, 3.6980575720599117`*^9}, 
   3.698057614928856*^9, 3.69805798427413*^9, 3.698058380170556*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{"Assuming", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"n", ">", "0"}], "&&", " ", 
     RowBox[{"1", "\[LessEqual]", " ", "i", "\[LessEqual]", " ", "n"}], " ", "&&",
      " ", 
     RowBox[{
     "1", " ", "\[LessEqual]", " ", "j", " ", "\[LessEqual]", " ", "n"}], " ",
      "&&", " ", 
     RowBox[{"p", " ", ">", " ", "0"}]}], ",", 
    RowBox[{"FullSimplify", "[", 
     RowBox[{
      UnderoverscriptBox["\[Sum]", 
       RowBox[{"i", "=", "1"}], "n"], 
      RowBox[{"Abs", "[", 
       RowBox[{
        FractionBox[
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], 
          RowBox[{"i", "+", "j"}]], 
         RowBox[{"p", "+", "i", "+", "j", "-", "1"}]], 
        FractionBox[
         RowBox[{"(", 
          RowBox[{
           UnderoverscriptBox["\[Product]", 
            RowBox[{"k", "=", "0"}], 
            RowBox[{"n", "-", "1"}]], 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"p", "+", "i", "+", "k"}], ")"}], 
            RowBox[{"(", 
             RowBox[{"p", "+", "j", "+", "k"}], ")"}]}]}], ")"}], 
         RowBox[{
          RowBox[{"Factorial", "[", 
           RowBox[{"i", "-", "1"}], "]"}], 
          RowBox[{"Factorial", "[", 
           RowBox[{"n", "-", "i"}], "]"}], 
          RowBox[{"Factorial", "[", 
           RowBox[{"j", "-", "1"}], "]"}], 
          RowBox[{"Factorial", "[", 
           RowBox[{"n", "-", "j"}], "]"}]}]]}], "]"}]}], "]"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.6980571716462812`*^9, 3.698057208234364*^9}, 
   3.698057240933728*^9, {3.698057648032028*^9, 3.69805772616562*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"Gamma", "[", 
    RowBox[{"1", "+", "n", "+", "p"}], "]"}], " ", 
   RowBox[{"Gamma", "[", 
    RowBox[{"j", "+", "n", "+", "p"}], "]"}], " ", 
   RowBox[{"HypergeometricPFQRegularized", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"1", "-", "n"}], ",", 
       RowBox[{"j", "+", "p"}], ",", 
       RowBox[{"1", "+", "n", "+", "p"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"1", "+", "p"}], ",", 
       RowBox[{"1", "+", "j", "+", "p"}]}], "}"}], ",", 
     RowBox[{"-", "1"}]}], "]"}]}], 
  RowBox[{
   RowBox[{"Gamma", "[", "j", "]"}], " ", 
   RowBox[{"Gamma", "[", "n", "]"}], " ", 
   RowBox[{"Gamma", "[", 
    RowBox[{"1", "-", "j", "+", "n"}], "]"}]}]]], "Output",
 CellChangeTimes->{3.6980577279938507`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionExpand", "[", 
  FractionBox[
   RowBox[{
    RowBox[{"Gamma", "[", 
     RowBox[{"1", "+", "n", "+", "p"}], "]"}], " ", 
    RowBox[{"Gamma", "[", 
     RowBox[{"j", "+", "n", "+", "p"}], "]"}], " ", 
    RowBox[{"HypergeometricPFQRegularized", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"1", "-", "n"}], ",", 
        RowBox[{"j", "+", "p"}], ",", 
        RowBox[{"1", "+", "n", "+", "p"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"1", "+", "p"}], ",", 
        RowBox[{"1", "+", "j", "+", "p"}]}], "}"}], ",", 
      RowBox[{"-", "1"}]}], "]"}]}], 
   RowBox[{
    RowBox[{"Gamma", "[", "j", "]"}], " ", 
    RowBox[{"Gamma", "[", "n", "]"}], " ", 
    RowBox[{"Gamma", "[", 
     RowBox[{"1", "-", "j", "+", "n"}], "]"}]}]], "]"}]], "Input",
 NumberMarks->False],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"Gamma", "[", 
    RowBox[{"1", "+", "n", "+", "p"}], "]"}], " ", 
   RowBox[{"Gamma", "[", 
    RowBox[{"j", "+", "n", "+", "p"}], "]"}], " ", 
   RowBox[{"HypergeometricPFQ", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"1", "-", "n"}], ",", 
       RowBox[{"j", "+", "p"}], ",", 
       RowBox[{"1", "+", "n", "+", "p"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"1", "+", "p"}], ",", 
       RowBox[{"1", "+", "j", "+", "p"}]}], "}"}], ",", 
     RowBox[{"-", "1"}]}], "]"}]}], 
  RowBox[{
   RowBox[{"Gamma", "[", "j", "]"}], " ", 
   RowBox[{"Gamma", "[", "n", "]"}], " ", 
   RowBox[{"Gamma", "[", 
    RowBox[{"1", "-", "j", "+", "n"}], "]"}], " ", 
   RowBox[{"Gamma", "[", 
    RowBox[{"1", "+", "p"}], "]"}], " ", 
   RowBox[{"Gamma", "[", 
    RowBox[{"1", "+", "j", "+", "p"}], "]"}]}]]], "Output",
 CellChangeTimes->{3.698057768386307*^9}]
}, Open  ]],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.698057906646903*^9, 3.698057943982037*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Assuming", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"n", ">", "0"}], "&&", " ", 
    RowBox[{"1", "\[LessEqual]", " ", "i", "\[LessEqual]", " ", "n"}], " ", "&&",
     " ", 
    RowBox[{
    "1", " ", "\[LessEqual]", " ", "j", " ", "\[LessEqual]", " ", "n"}], " ", 
    "&&", " ", 
    RowBox[{"p", " ", ">", " ", "0"}]}], ",", 
   RowBox[{"FullSimplify", "[", 
    RowBox[{
     UnderoverscriptBox["\[Sum]", 
      RowBox[{"i", "=", "1"}], "n"], 
     RowBox[{"Abs", "[", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"p", "+", "i", "+", "j", "-", "1"}], ")"}], 
       RowBox[{"-", "1"}]], "]"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.698058290394226*^9, 3.6980583016047497`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   RowBox[{"PolyGamma", "[", 
    RowBox[{"0", ",", 
     RowBox[{"j", "+", "p"}]}], "]"}]}], "+", 
  RowBox[{"PolyGamma", "[", 
   RowBox[{"0", ",", 
    RowBox[{"j", "+", "n", "+", "p"}]}], "]"}]}]], "Output",
 CellChangeTimes->{3.698058314198155*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PolyGamma", "[", "3", "]"}]], "Input",
 CellChangeTimes->{{3.698058514586821*^9, 3.6980585193133097`*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["3", "2"], "-", "EulerGamma"}]], "Output",
 CellChangeTimes->{3.69805851975422*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PolyGamma", "[", 
  RowBox[{"0", ",", "3"}], "]"}]], "Input",
 CellChangeTimes->{{3.698058522212612*^9, 3.698058525508246*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["3", "2"], "-", "EulerGamma"}]], "Output",
 CellChangeTimes->{3.698058525989798*^9}]
}, Open  ]]
},
WindowSize->{1244, 616},
WindowMargins->{{408, Automatic}, {Automatic, 160}},
FrontEndVersion->"11.0 for Linux x86 (64-bit) (July 28, 2016)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[580, 22, 5123, 151, 453, "Input"],
Cell[5706, 175, 1034, 30, 51, "Output"],
Cell[6743, 207, 872, 22, 32, "Output"],
Cell[7618, 231, 391, 7, 32, "Output"],
Cell[8012, 240, 391, 7, 32, "Output"],
Cell[8406, 249, 474, 10, 51, "Output"],
Cell[8883, 261, 473, 10, 51, "Output"],
Cell[9359, 273, 940, 23, 102, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[10336, 301, 1669, 46, 129, "Input"],
Cell[12008, 349, 816, 24, 55, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[12861, 378, 855, 25, 69, "Input"],
Cell[13719, 405, 949, 28, 88, "Output"]
}, Open  ]],
Cell[14683, 436, 92, 1, 30, "Input"],
Cell[CellGroupData[{
Cell[14800, 441, 732, 20, 63, "Input"],
Cell[15535, 463, 293, 9, 32, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[15865, 477, 130, 2, 32, "Input"],
Cell[15998, 481, 121, 3, 51, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[16156, 489, 151, 3, 32, "Input"],
Cell[16310, 494, 122, 3, 84, "Output"]
}, Open  ]]
}
]
*)

(* End of internal cache information *)

